@@ -33,7 +33,6 @@
 #include <stdio.h>
 
 #include "avcodec.h"
-#include "internal.h"
 #include "get_bits.h"
 #include "dsputil.h"
 #include "fft.h"
@@ -247,7 +246,7 @@ static int at1_unpack_dequant(GetBitContext* gb, AT1SUCtx* su,
                      */
                     spec[pos+i] = get_sbits(gb, word_len) * scale_factor * max_quant;
                 }
-            } else { /* word_len = 0 -> empty BFU, zero all specs in the emty BFU */
+            } else { /* word_len = 0 -> empty BFU, zero all specs in the empty BFU */
                 memset(&spec[pos], 0, num_specs * sizeof(float));
             }
         }
@@ -263,14 +262,14 @@ static void at1_subband_synthesis(AT1Ctx *q, AT1SUCtx* su, float *pOut)
     float iqmf_temp[512 + 46];
 
     /* combine low and middle bands */
-    atrac_iqmf(q->bands[0], q->bands[1], 128, temp, su->fst_qmf_delay, iqmf_temp);
+    ff_atrac_iqmf(q->bands[0], q->bands[1], 128, temp, su->fst_qmf_delay, iqmf_temp);
 
     /* delay the signal of the high band by 23 samples */
     memcpy( su->last_qmf_delay,    &su->last_qmf_delay[256], sizeof(float) *  23);
     memcpy(&su->last_qmf_delay[23], q->bands[2],             sizeof(float) * 256);
 
     /* combine (low + middle) and high bands */
-    atrac_iqmf(temp, su->last_qmf_delay, 256, pOut, su->snd_qmf_delay, iqmf_temp);
+    ff_atrac_iqmf(temp, su->last_qmf_delay, 256, pOut, su->snd_qmf_delay, iqmf_temp);
 }
 
 
@@ -292,7 +291,7 @@ static int atrac1_decode_frame(AVCodecContext *avctx, void *data,
 
     /* get output buffer */
     q->frame.nb_samples = AT1_SU_SAMPLES;
-    if ((ret = ff_get_buffer(avctx, &q->frame)) < 0) {
+    if ((ret = avctx->get_buffer(avctx, &q->frame)) < 0) {
         av_log(avctx, AV_LOG_ERROR, "get_buffer() failed\n");
         return ret;
     }
@@ -379,9 +378,9 @@ static av_cold int atrac1_decode_init(AVCodecContext *avctx)
 
     ff_init_ff_sine_windows(5);
 
-    atrac_generate_tables();
+    ff_atrac_generate_tables();
 
-    dsputil_init(&q->dsp, avctx);
+    ff_dsputil_init(&q->dsp, avctx);
     ff_fmt_convert_init(&q->fmt_conv, avctx);
 
     q->bands[0] = q->low;
@@ -402,13 +401,13 @@ static av_cold int atrac1_decode_init(AVCodecContext *avctx)
 
 
 AVCodec ff_atrac1_decoder = {
-    .name = "atrac1",
-    .type = AVMEDIA_TYPE_AUDIO,
-    .id = CODEC_ID_ATRAC1,
+    .name           = "atrac1",
+    .type           = AVMEDIA_TYPE_AUDIO,
+    .id             = AV_CODEC_ID_ATRAC1,
     .priv_data_size = sizeof(AT1Ctx),
-    .init = atrac1_decode_init,
-    .close = atrac1_decode_end,
-    .decode = atrac1_decode_frame,
-    .capabilities = CODEC_CAP_DR1,
-    .long_name = NULL_IF_CONFIG_SMALL("Atrac 1 (Adaptive TRansform Acoustic Coding)"),
+    .init           = atrac1_decode_init,
+    .close          = atrac1_decode_end,
+    .decode         = atrac1_decode_frame,
+    .capabilities   = CODEC_CAP_DR1,
+    .long_name      = NULL_IF_CONFIG_SMALL("Atrac 1 (Adaptive TRansform Acoustic Coding)"),
 };
