@@ -26,6 +26,7 @@
 
 #include "avcodec.h"
 #include "raw.h"
+#include "libavutil/common.h"
 
 const PixelFormatTag ff_raw_pix_fmt_tags[] = {
     { PIX_FMT_YUV420P, MKTAG('I', '4', '2', '0') }, /* Planar formats */
@@ -63,6 +64,7 @@ const PixelFormatTag ff_raw_pix_fmt_tags[] = {
     { PIX_FMT_UYVY422, MKTAG('A', 'V', 'u', 'p') },
     { PIX_FMT_UYVY422, MKTAG('V', 'D', 'T', 'Z') }, /* SoftLab-NSK VideoTizer */
     { PIX_FMT_UYVY422, MKTAG('a', 'u', 'v', '2') },
+    { PIX_FMT_UYVY422, MKTAG('c', 'y', 'u', 'v') }, /* CYUV is also Creative YUV */
     { PIX_FMT_UYYVYY411, MKTAG('Y', '4', '1', '1') },
     { PIX_FMT_GRAY8,   MKTAG('G', 'R', 'E', 'Y') },
     { PIX_FMT_NV12,    MKTAG('N', 'V', '1', '2') },
@@ -81,10 +83,18 @@ const PixelFormatTag ff_raw_pix_fmt_tags[] = {
     { PIX_FMT_BGR444LE, MKTAG('B', 'G', 'R', 12) },
     { PIX_FMT_RGB444BE, MKTAG(12 , 'B', 'G', 'R') },
     { PIX_FMT_BGR444BE, MKTAG(12 , 'R', 'G', 'B') },
+    { PIX_FMT_RGBA64LE, MKTAG('R', 'B', 'A', 64 ) },
+    { PIX_FMT_BGRA64LE, MKTAG('B', 'R', 'A', 64 ) },
+    { PIX_FMT_RGBA64BE, MKTAG(64 , 'R', 'B', 'A') },
+    { PIX_FMT_BGRA64BE, MKTAG(64 , 'B', 'R', 'A') },
     { PIX_FMT_RGBA,     MKTAG('R', 'G', 'B', 'A') },
+    { PIX_FMT_RGB0,     MKTAG('R', 'G', 'B',  0 ) },
     { PIX_FMT_BGRA,     MKTAG('B', 'G', 'R', 'A') },
+    { PIX_FMT_BGR0,     MKTAG('B', 'G', 'R',  0 ) },
     { PIX_FMT_ABGR,     MKTAG('A', 'B', 'G', 'R') },
+    { PIX_FMT_0BGR,     MKTAG( 0 , 'B', 'G', 'R') },
     { PIX_FMT_ARGB,     MKTAG('A', 'R', 'G', 'B') },
+    { PIX_FMT_0RGB,     MKTAG( 0 , 'R', 'G', 'B') },
     { PIX_FMT_RGB24,    MKTAG('R', 'G', 'B', 24 ) },
     { PIX_FMT_BGR24,    MKTAG('B', 'G', 'R', 24 ) },
     { PIX_FMT_YUV411P,  MKTAG('4', '1', '1', 'P') },
@@ -114,6 +124,18 @@ const PixelFormatTag ff_raw_pix_fmt_tags[] = {
     { PIX_FMT_YUV422P10BE, MKTAG(10 , 10 , '3', 'Y') },
     { PIX_FMT_YUV444P10LE, MKTAG('Y', '3',  0 , 10 ) },
     { PIX_FMT_YUV444P10BE, MKTAG(10 ,  0 , '3', 'Y') },
+    { PIX_FMT_YUV420P12LE, MKTAG('Y', '3', 11 , 12 ) },
+    { PIX_FMT_YUV420P12BE, MKTAG(12 , 11 , '3', 'Y') },
+    { PIX_FMT_YUV422P12LE, MKTAG('Y', '3', 10 , 12 ) },
+    { PIX_FMT_YUV422P12BE, MKTAG(12 , 10 , '3', 'Y') },
+    { PIX_FMT_YUV444P12LE, MKTAG('Y', '3',  0 , 12 ) },
+    { PIX_FMT_YUV444P12BE, MKTAG(12 ,  0 , '3', 'Y') },
+    { PIX_FMT_YUV420P14LE, MKTAG('Y', '3', 11 , 14 ) },
+    { PIX_FMT_YUV420P14BE, MKTAG(14 , 11 , '3', 'Y') },
+    { PIX_FMT_YUV422P14LE, MKTAG('Y', '3', 10 , 14 ) },
+    { PIX_FMT_YUV422P14BE, MKTAG(14 , 10 , '3', 'Y') },
+    { PIX_FMT_YUV444P14LE, MKTAG('Y', '3',  0 , 14 ) },
+    { PIX_FMT_YUV444P14BE, MKTAG(14 ,  0 , '3', 'Y') },
     { PIX_FMT_YUV420P16LE, MKTAG('Y', '3', 11 , 16 ) },
     { PIX_FMT_YUV420P16BE, MKTAG(16 , 11 , '3', 'Y') },
     { PIX_FMT_YUV422P16LE, MKTAG('Y', '3', 10 , 16 ) },
@@ -121,12 +143,17 @@ const PixelFormatTag ff_raw_pix_fmt_tags[] = {
     { PIX_FMT_YUV444P16LE, MKTAG('Y', '3',  0 , 16 ) },
     { PIX_FMT_YUV444P16BE, MKTAG(16 ,  0 , '3', 'Y') },
     { PIX_FMT_YUVA420P,    MKTAG('Y', '4', 11 ,  8 ) },
+    { PIX_FMT_YUVA422P,    MKTAG('Y', '4', 10 ,  8 ) },
+    { PIX_FMT_YUVA444P,    MKTAG('Y', '4',  0 ,  8 ) },
     { PIX_FMT_GRAY8A,      MKTAG('Y', '2',  0 ,  8 ) },
 
     /* quicktime */
+    { PIX_FMT_YUV420P, MKTAG('R', '4', '2', '0') }, /* Radius DV YUV PAL */
+    { PIX_FMT_YUV411P, MKTAG('R', '4', '1', '1') }, /* Radius DV YUV NTSC */
     { PIX_FMT_UYVY422, MKTAG('2', 'v', 'u', 'y') },
     { PIX_FMT_UYVY422, MKTAG('2', 'V', 'u', 'y') },
     { PIX_FMT_UYVY422, MKTAG('A', 'V', 'U', 'I') }, /* FIXME merge both fields */
+    { PIX_FMT_UYVY422, MKTAG('b', 'x', 'y', 'v') },
     { PIX_FMT_YUYV422, MKTAG('y', 'u', 'v', '2') },
     { PIX_FMT_YUYV422, MKTAG('y', 'u', 'v', 's') },
     { PIX_FMT_YUYV422, MKTAG('D', 'V', 'O', 'O') }, /* Digital Voodoo SD 8 Bit */
@@ -134,8 +161,10 @@ const PixelFormatTag ff_raw_pix_fmt_tags[] = {
     { PIX_FMT_RGB565LE,MKTAG('L', '5', '6', '5') },
     { PIX_FMT_RGB565BE,MKTAG('B', '5', '6', '5') },
     { PIX_FMT_BGR24,   MKTAG('2', '4', 'B', 'G') },
+    { PIX_FMT_BGR24,   MKTAG('b', 'x', 'b', 'g') },
     { PIX_FMT_BGRA,    MKTAG('B', 'G', 'R', 'A') },
     { PIX_FMT_RGBA,    MKTAG('R', 'G', 'B', 'A') },
+    { PIX_FMT_RGB24,   MKTAG('b', 'x', 'r', 'g') },
     { PIX_FMT_ABGR,    MKTAG('A', 'B', 'G', 'R') },
     { PIX_FMT_GRAY16BE,MKTAG('b', '1', '6', 'g') },
     { PIX_FMT_RGB48BE, MKTAG('b', '4', '8', 'r') },
@@ -143,6 +172,7 @@ const PixelFormatTag ff_raw_pix_fmt_tags[] = {
     /* special */
     { PIX_FMT_RGB565LE,MKTAG( 3 ,  0 ,  0 ,  0 ) }, /* flipped RGB565LE */
     { PIX_FMT_YUV444P, MKTAG('Y', 'V', '2', '4') }, /* YUV444P, swapped UV */
+    { PIX_FMT_YUYV422, MKTAG('Y', 'V', 'Y', 'U') }, /* YUYV, swapped UV */
 
     { PIX_FMT_NONE, 0 },
 };
