@@ -113,18 +113,13 @@ static int kset(AVFormatContext *s, const uint8_t *r_val, const uint8_t *n_val,
     return 0;
 }
 
-#define OMA_RPROBE_M_VAL 48 + 1
-
-static int rprobe(AVFormatContext *s, uint8_t *enc_header, unsigned size,
-                  const uint8_t *r_val)
+static int rprobe(AVFormatContext *s, uint8_t *enc_header, const uint8_t *r_val)
 {
     OMAContext *oc = s->priv_data;
     unsigned int pos;
     struct AVDES av_des;
 
-    if (!enc_header || !r_val ||
-        size < OMA_ENC_HEADER_SIZE + oc->k_size + oc->e_size + oc->i_size ||
-        size < OMA_RPROBE_M_VAL)
+    if (!enc_header || !r_val)
         return -1;
 
     /* m_val */
@@ -145,25 +140,19 @@ static int rprobe(AVFormatContext *s, uint8_t *enc_header, unsigned size,
     return memcmp(&enc_header[pos], oc->sm_val, 8) ? -1 : 0;
 }
 
-static int nprobe(AVFormatContext *s, uint8_t *enc_header, unsigned size,
-                  const uint8_t *n_val)
+static int nprobe(AVFormatContext *s, uint8_t *enc_header, int size, const uint8_t *n_val)
 {
     OMAContext *oc = s->priv_data;
-    uint64_t pos;
-    uint32_t taglen, datalen;
+    uint32_t pos, taglen, datalen;
     struct AVDES av_des;
 
-    if (!enc_header || !n_val ||
-        size < OMA_ENC_HEADER_SIZE + oc->k_size + 4)
+    if (!enc_header || !n_val)
         return -1;
 
     pos = OMA_ENC_HEADER_SIZE + oc->k_size;
     if (!memcmp(&enc_header[pos], "EKB ", 4))
         pos += 32;
 
-    if (size < pos + 44)
-        return -1;
-
     if (AV_RB32(&enc_header[pos]) != oc->rid)
         av_log(s, AV_LOG_DEBUG, "Mismatching RID\n");
 
@@ -173,20 +162,13 @@ static int nprobe(AVFormatContext *s, uint8_t *enc_header, unsigned size,
     if(taglen + (((uint64_t)datalen)<<4) + 44 > size)
         return -1;
 
-    pos += 44;
-    if (size - pos < taglen)
-        return -1;
-
-    pos += taglen;
-
-    if (datalen << 4 > size - pos)
-        return -1;
+    pos += 44 + taglen;
 
     av_des_init(&av_des, n_val, 192, 1);
     while (datalen-- > 0) {
         av_des_crypt(&av_des, oc->r_val, &enc_header[pos], 2, NULL, 1);
         kset(s, oc->r_val, NULL, 16);
-        if (!rprobe(s, enc_header, size, oc->r_val))
+        if (!rprobe(s, enc_header, oc->r_val))
             return 0;
         pos += 16;
     }
@@ -237,6 +219,12 @@ static int decrypt_init(AVFormatContext *s, ID3v2ExtraMeta *em, uint8_t *header)
         av_log(s, AV_LOG_ERROR, "Invalid encryption header\n");
         return -1;
     }
+    if (   OMA_ENC_HEADER_SIZE + oc->k_size + oc->e_size + oc->i_size + 8 > geob->datasize
+        || OMA_ENC_HEADER_SIZE + 48 > geob->datasize
+    ) {
+        av_log(s, AV_LOG_ERROR, "Too little GEOB data\n");
+        return AVERROR_INVALIDDATA;
+    }
     oc->rid = AV_RB32(&gdata[OMA_ENC_HEADER_SIZE + 28]);
     av_log(s, AV_LOG_DEBUG, "RID: %.8x\n", oc->rid);
 
@@ -249,7 +237,7 @@ static int decrypt_init(AVFormatContext *s, ID3v2ExtraMeta *em, uint8_t *header)
         kset(s, s->key, s->key, s->keylen);
     }
     if (!memcmp(oc->r_val, (const uint8_t[8]){0}, 8) ||
-        rprobe(s, gdata, geob->datasize, oc->r_val) < 0 &&
+        rprobe(s, gdata, oc->r_val) < 0 &&
         nprobe(s, gdata, geob->datasize, oc->n_val) < 0) {
         int i;
         for (i = 0; i < FF_ARRAY_ELEMS(leaf_table); i += 2) {
@@ -257,11 +245,10 @@ static int decrypt_init(AVFormatContext *s, ID3v2ExtraMeta *em, uint8_t *header)
             AV_WL64(buf, leaf_table[i]);
             AV_WL64(&buf[8], leaf_table[i+1]);
             kset(s, buf, buf, 16);
-            if (!rprobe(s, gdata, geob->datasize, oc->r_val) ||
-                !nprobe(s, gdata, geob->datasize, oc->n_val))
+            if (!rprobe(s, gdata, oc->r_val) || !nprobe(s, gdata, geob->datasize, oc->n_val))
                 break;
         }
-        if (i >= sizeof(leaf_table)) {
+        if (i >= FF_ARRAY_ELEMS(leaf_table)) {
             av_log(s, AV_LOG_ERROR, "Invalid key\n");
             return -1;
         }
@@ -278,8 +265,7 @@ static int decrypt_init(AVFormatContext *s, ID3v2ExtraMeta *em, uint8_t *header)
     return 0;
 }
 
-static int oma_read_header(AVFormatContext *s,
-                           AVFormatParameters *ap)
+static int oma_read_header(AVFormatContext *s)
 {
     int     ret, framesize, jsflag, samplerate;
     uint32_t codec_params;
@@ -290,7 +276,7 @@ static int oma_read_header(AVFormatContext *s,
     ID3v2ExtraMeta *extra_meta = NULL;
     OMAContext *oc = s->priv_data;
 
-    ff_id3v2_read_all(s, ID3v2_EA3_MAGIC, &extra_meta);
+    ff_id3v2_read(s, ID3v2_EA3_MAGIC, &extra_meta);
     ret = avio_read(s->pb, buf, EA3_HEADER_SIZE);
     if (ret < EA3_HEADER_SIZE)
         return -1;
@@ -324,11 +310,7 @@ static int oma_read_header(AVFormatContext *s,
 
     switch (buf[32]) {
         case OMA_CODECID_ATRAC3:
-            samplerate = ff_oma_srate_tab[(codec_params >> 13) & 7] * 100;
-            if (!samplerate) {
-                av_log(s, AV_LOG_ERROR, "Unsupported sample rate\n");
-                return AVERROR_INVALIDDATA;
-            }
+            samplerate = ff_oma_srate_tab[(codec_params >> 13) & 7]*100;
             if (samplerate != 44100)
                 av_log_ask_for_sample(s, "Unsupported sample rate: %d\n",
                                       samplerate);
@@ -358,18 +340,13 @@ static int oma_read_header(AVFormatContext *s,
         case OMA_CODECID_ATRAC3P:
             st->codec->channels = (codec_params >> 10) & 7;
             framesize = ((codec_params & 0x3FF) * 8) + 8;
-            samplerate = ff_oma_srate_tab[(codec_params >> 13) & 7] * 100;
-            if (!samplerate) {
-                av_log(s, AV_LOG_ERROR, "Unsupported sample rate\n");
-                return AVERROR_INVALIDDATA;
-            }
-            st->codec->sample_rate = samplerate;
-            st->codec->bit_rate    = samplerate * framesize * 8 / 1024;
-            avpriv_set_pts_info(st, 64, 1, samplerate);
+            st->codec->sample_rate = ff_oma_srate_tab[(codec_params >> 13) & 7]*100;
+            st->codec->bit_rate    = st->codec->sample_rate * framesize * 8 / 1024;
+            avpriv_set_pts_info(st, 64, 1, st->codec->sample_rate);
             av_log(s, AV_LOG_ERROR, "Unsupported codec ATRAC3+!\n");
             break;
         case OMA_CODECID_MP3:
-            st->need_parsing = AVSTREAM_PARSE_FULL;
+            st->need_parsing = AVSTREAM_PARSE_FULL_RAW;
             framesize = 1024;
             break;
         case OMA_CODECID_LPCM:
@@ -399,22 +376,14 @@ static int oma_read_packet(AVFormatContext *s, AVPacket *pkt)
     int packet_size = s->streams[0]->codec->block_align;
     int ret = av_get_packet(s->pb, pkt, packet_size);
 
-    if (ret < packet_size)
-        pkt->flags |= AV_PKT_FLAG_CORRUPT;
-
     if (ret <= 0)
         return AVERROR(EIO);
 
     pkt->stream_index = 0;
 
     if (oc->encrypted) {
-        /* previous unencrypted block saved in IV for
-         * the next packet (CBC mode) */
-        if (ret == packet_size)
-            av_des_crypt(&oc->av_des, pkt->data, pkt->data,
-                         (packet_size >> 3), oc->iv, 1);
-        else
-            memset(oc->iv, 0, 8);
+        /* previous unencrypted block saved in IV for the next packet (CBC mode) */
+        av_des_crypt(&oc->av_des, pkt->data, pkt->data, (ret >> 3), oc->iv, 1);
     }
 
     return ret;
@@ -452,26 +421,22 @@ static int oma_read_seek(struct AVFormatContext *s, int stream_index, int64_t ti
 {
     OMAContext *oc = s->priv_data;
 
-    int err = pcm_read_seek(s, stream_index, timestamp, flags);
+    ff_pcm_read_seek(s, stream_index, timestamp, flags);
 
-    if (!oc->encrypted)
-        return err;
-
-    /* readjust IV for CBC */
-    if (err || avio_tell(s->pb) < oc->content_start)
-        goto wipe;
-    if ((err = avio_seek(s->pb, -8, SEEK_CUR)) < 0)
-        goto wipe;
-    if ((err = avio_read(s->pb, oc->iv, 8)) < 8) {
-        if (err >= 0)
-            err = AVERROR_EOF;
-        goto wipe;
+    if (oc->encrypted) {
+        /* readjust IV for CBC */
+        int64_t pos = avio_tell(s->pb);
+        if (pos < oc->content_start)
+            memset(oc->iv, 0, 8);
+        else {
+            if (avio_seek(s->pb, -8, SEEK_CUR) < 0 || avio_read(s->pb, oc->iv, 8) < 8) {
+                memset(oc->iv, 0, 8);
+                return -1;
+            }
+        }
     }
 
     return 0;
-wipe:
-    memset(oc->iv, 0, 8);
-    return err;
 }
 
 AVInputFormat ff_oma_demuxer = {
@@ -486,4 +451,3 @@ AVInputFormat ff_oma_demuxer = {
     .extensions     = "oma,omg,aa3",
     .codec_tag      = (const AVCodecTag* const []){ff_oma_codec_tags, 0},
 };
-
